/****************************************************************************
** Meta object code from reading C++ file 'mainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../speedcrunch/src/gui/mainwindow.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_MainWindow_t {
    QByteArrayData data[116];
    char stringdata0[2384];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MainWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MainWindow_t qt_meta_stringdata_MainWindow = {
    {
QT_MOC_LITERAL(0, 0, 10), // "MainWindow"
QT_MOC_LITERAL(1, 11, 16), // "angleUnitChanged"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 18), // "colorSchemeChanged"
QT_MOC_LITERAL(4, 48, 15), // "languageChanged"
QT_MOC_LITERAL(5, 64, 21), // "radixCharacterChanged"
QT_MOC_LITERAL(6, 86, 19), // "resultFormatChanged"
QT_MOC_LITERAL(7, 106, 22), // "resultPrecisionChanged"
QT_MOC_LITERAL(8, 129, 25), // "syntaxHighlightingChanged"
QT_MOC_LITERAL(9, 155, 14), // "historyChanged"
QT_MOC_LITERAL(10, 170, 16), // "variablesChanged"
QT_MOC_LITERAL(11, 187, 16), // "functionsChanged"
QT_MOC_LITERAL(12, 204, 4), // "copy"
QT_MOC_LITERAL(13, 209, 24), // "applySelectedColorScheme"
QT_MOC_LITERAL(14, 234, 11), // "clearEditor"
QT_MOC_LITERAL(15, 246, 22), // "clearEditorAndBitfield"
QT_MOC_LITERAL(16, 269, 12), // "clearHistory"
QT_MOC_LITERAL(17, 282, 21), // "copyResultToClipboard"
QT_MOC_LITERAL(18, 304, 15), // "cycleAngleUnits"
QT_MOC_LITERAL(19, 320, 18), // "cycleResultFormats"
QT_MOC_LITERAL(20, 339, 28), // "decreaseDisplayFontPointSize"
QT_MOC_LITERAL(21, 368, 15), // "decreaseOpacity"
QT_MOC_LITERAL(22, 384, 15), // "deleteVariables"
QT_MOC_LITERAL(23, 400, 19), // "deleteUserFunctions"
QT_MOC_LITERAL(24, 420, 24), // "evaluateEditorExpression"
QT_MOC_LITERAL(25, 445, 10), // "exportHtml"
QT_MOC_LITERAL(26, 456, 15), // "exportPlainText"
QT_MOC_LITERAL(27, 472, 30), // "handleAutoCalcMessageAvailable"
QT_MOC_LITERAL(28, 503, 31), // "handleAutoCalcQuantityAvailable"
QT_MOC_LITERAL(29, 535, 8), // "Quantity"
QT_MOC_LITERAL(30, 544, 19), // "handleCopyAvailable"
QT_MOC_LITERAL(31, 564, 13), // "copyAvailable"
QT_MOC_LITERAL(32, 578, 17), // "handleBitsChanged"
QT_MOC_LITERAL(33, 596, 23), // "handleKeypadButtonPress"
QT_MOC_LITERAL(34, 620, 14), // "Keypad::Button"
QT_MOC_LITERAL(35, 635, 22), // "handleEditorTextChange"
QT_MOC_LITERAL(36, 658, 28), // "handleDisplaySelectionChange"
QT_MOC_LITERAL(37, 687, 27), // "handleEditorSelectionChange"
QT_MOC_LITERAL(38, 715, 18), // "handleManualClosed"
QT_MOC_LITERAL(39, 734, 33), // "handleDockWidgetVisibilityCha..."
QT_MOC_LITERAL(40, 768, 7), // "visible"
QT_MOC_LITERAL(41, 776, 14), // "hideStateLabel"
QT_MOC_LITERAL(42, 791, 28), // "increaseDisplayFontPointSize"
QT_MOC_LITERAL(43, 820, 15), // "increaseOpacity"
QT_MOC_LITERAL(44, 836, 24), // "insertConstantIntoEditor"
QT_MOC_LITERAL(45, 861, 24), // "insertFunctionIntoEditor"
QT_MOC_LITERAL(46, 886, 20), // "insertTextIntoEditor"
QT_MOC_LITERAL(47, 907, 24), // "insertVariableIntoEditor"
QT_MOC_LITERAL(48, 932, 28), // "insertUserFunctionIntoEditor"
QT_MOC_LITERAL(49, 961, 14), // "openUpdatesURL"
QT_MOC_LITERAL(50, 976, 15), // "openFeedbackURL"
QT_MOC_LITERAL(51, 992, 16), // "openCommunityURL"
QT_MOC_LITERAL(52, 1009, 11), // "openNewsURL"
QT_MOC_LITERAL(53, 1021, 13), // "openDonateURL"
QT_MOC_LITERAL(54, 1035, 15), // "retranslateText"
QT_MOC_LITERAL(55, 1051, 17), // "saveSessionDialog"
QT_MOC_LITERAL(56, 1069, 22), // "selectEditorExpression"
QT_MOC_LITERAL(57, 1092, 21), // "setAlwaysOnTopEnabled"
QT_MOC_LITERAL(58, 1114, 18), // "setAngleModeDegree"
QT_MOC_LITERAL(59, 1133, 18), // "setAngleModeRadian"
QT_MOC_LITERAL(60, 1152, 17), // "setAutoAnsEnabled"
QT_MOC_LITERAL(61, 1170, 18), // "setAutoCalcEnabled"
QT_MOC_LITERAL(62, 1189, 24), // "setAutoCompletionEnabled"
QT_MOC_LITERAL(63, 1214, 18), // "setBitfieldVisible"
QT_MOC_LITERAL(64, 1233, 23), // "setConstantsDockVisible"
QT_MOC_LITERAL(65, 1257, 9), // "takeFocus"
QT_MOC_LITERAL(66, 1267, 25), // "setFormulaBookDockVisible"
QT_MOC_LITERAL(67, 1293, 20), // "setFullScreenEnabled"
QT_MOC_LITERAL(68, 1314, 23), // "setFunctionsDockVisible"
QT_MOC_LITERAL(69, 1338, 21), // "setHistoryDockVisible"
QT_MOC_LITERAL(70, 1360, 21), // "setSessionSaveEnabled"
QT_MOC_LITERAL(71, 1382, 16), // "setKeypadVisible"
QT_MOC_LITERAL(72, 1399, 29), // "setLeaveLastExpressionEnabled"
QT_MOC_LITERAL(73, 1429, 26), // "setRadixCharacterAutomatic"
QT_MOC_LITERAL(74, 1456, 17), // "setRadixCharacter"
QT_MOC_LITERAL(75, 1474, 22), // "setRadixCharacterComma"
QT_MOC_LITERAL(76, 1497, 20), // "setRadixCharacterDot"
QT_MOC_LITERAL(77, 1518, 21), // "setRadixCharacterBoth"
QT_MOC_LITERAL(78, 1540, 21), // "setResultFormatBinary"
QT_MOC_LITERAL(79, 1562, 24), // "setResultFormatCartesian"
QT_MOC_LITERAL(80, 1587, 15), // "setResultFormat"
QT_MOC_LITERAL(81, 1603, 26), // "setResultFormatEngineering"
QT_MOC_LITERAL(82, 1630, 20), // "setResultFormatFixed"
QT_MOC_LITERAL(83, 1651, 22), // "setResultFormatGeneral"
QT_MOC_LITERAL(84, 1674, 26), // "setResultFormatHexadecimal"
QT_MOC_LITERAL(85, 1701, 20), // "setResultFormatOctal"
QT_MOC_LITERAL(86, 1722, 20), // "setResultFormatPolar"
QT_MOC_LITERAL(87, 1743, 25), // "setResultFormatScientific"
QT_MOC_LITERAL(88, 1769, 26), // "setResultPrecision15Digits"
QT_MOC_LITERAL(89, 1796, 25), // "setResultPrecision2Digits"
QT_MOC_LITERAL(90, 1822, 25), // "setResultPrecision3Digits"
QT_MOC_LITERAL(91, 1848, 26), // "setResultPrecision50Digits"
QT_MOC_LITERAL(92, 1875, 25), // "setResultPrecision8Digits"
QT_MOC_LITERAL(93, 1901, 27), // "setResultPrecisionAutomatic"
QT_MOC_LITERAL(94, 1929, 18), // "setResultPrecision"
QT_MOC_LITERAL(95, 1948, 19), // "setStatusBarVisible"
QT_MOC_LITERAL(96, 1968, 28), // "setSyntaxHighlightingEnabled"
QT_MOC_LITERAL(97, 1997, 16), // "setDigitGrouping"
QT_MOC_LITERAL(98, 2014, 8), // "QAction*"
QT_MOC_LITERAL(99, 2023, 31), // "setAutoResultToClipboardEnabled"
QT_MOC_LITERAL(100, 2055, 17), // "setComplexNumbers"
QT_MOC_LITERAL(101, 2073, 23), // "setVariablesDockVisible"
QT_MOC_LITERAL(102, 2097, 27), // "setUserFunctionsDockVisible"
QT_MOC_LITERAL(103, 2125, 28), // "setWindowPositionSaveEnabled"
QT_MOC_LITERAL(104, 2154, 19), // "setWidgetsDirection"
QT_MOC_LITERAL(105, 2174, 15), // "showAboutDialog"
QT_MOC_LITERAL(106, 2190, 14), // "showStateLabel"
QT_MOC_LITERAL(107, 2205, 14), // "showFontDialog"
QT_MOC_LITERAL(108, 2220, 25), // "showLanguageChooserDialog"
QT_MOC_LITERAL(109, 2246, 16), // "showManualWindow"
QT_MOC_LITERAL(110, 2263, 15), // "showContextHelp"
QT_MOC_LITERAL(111, 2279, 16), // "showReadyMessage"
QT_MOC_LITERAL(112, 2296, 27), // "showResultFormatContextMenu"
QT_MOC_LITERAL(113, 2324, 23), // "showSessionImportDialog"
QT_MOC_LITERAL(114, 2348, 21), // "showSessionLoadDialog"
QT_MOC_LITERAL(115, 2370, 13) // "wrapSelection"

    },
    "MainWindow\0angleUnitChanged\0\0"
    "colorSchemeChanged\0languageChanged\0"
    "radixCharacterChanged\0resultFormatChanged\0"
    "resultPrecisionChanged\0syntaxHighlightingChanged\0"
    "historyChanged\0variablesChanged\0"
    "functionsChanged\0copy\0applySelectedColorScheme\0"
    "clearEditor\0clearEditorAndBitfield\0"
    "clearHistory\0copyResultToClipboard\0"
    "cycleAngleUnits\0cycleResultFormats\0"
    "decreaseDisplayFontPointSize\0"
    "decreaseOpacity\0deleteVariables\0"
    "deleteUserFunctions\0evaluateEditorExpression\0"
    "exportHtml\0exportPlainText\0"
    "handleAutoCalcMessageAvailable\0"
    "handleAutoCalcQuantityAvailable\0"
    "Quantity\0handleCopyAvailable\0copyAvailable\0"
    "handleBitsChanged\0handleKeypadButtonPress\0"
    "Keypad::Button\0handleEditorTextChange\0"
    "handleDisplaySelectionChange\0"
    "handleEditorSelectionChange\0"
    "handleManualClosed\0handleDockWidgetVisibilityChanged\0"
    "visible\0hideStateLabel\0"
    "increaseDisplayFontPointSize\0"
    "increaseOpacity\0insertConstantIntoEditor\0"
    "insertFunctionIntoEditor\0insertTextIntoEditor\0"
    "insertVariableIntoEditor\0"
    "insertUserFunctionIntoEditor\0"
    "openUpdatesURL\0openFeedbackURL\0"
    "openCommunityURL\0openNewsURL\0openDonateURL\0"
    "retranslateText\0saveSessionDialog\0"
    "selectEditorExpression\0setAlwaysOnTopEnabled\0"
    "setAngleModeDegree\0setAngleModeRadian\0"
    "setAutoAnsEnabled\0setAutoCalcEnabled\0"
    "setAutoCompletionEnabled\0setBitfieldVisible\0"
    "setConstantsDockVisible\0takeFocus\0"
    "setFormulaBookDockVisible\0"
    "setFullScreenEnabled\0setFunctionsDockVisible\0"
    "setHistoryDockVisible\0setSessionSaveEnabled\0"
    "setKeypadVisible\0setLeaveLastExpressionEnabled\0"
    "setRadixCharacterAutomatic\0setRadixCharacter\0"
    "setRadixCharacterComma\0setRadixCharacterDot\0"
    "setRadixCharacterBoth\0setResultFormatBinary\0"
    "setResultFormatCartesian\0setResultFormat\0"
    "setResultFormatEngineering\0"
    "setResultFormatFixed\0setResultFormatGeneral\0"
    "setResultFormatHexadecimal\0"
    "setResultFormatOctal\0setResultFormatPolar\0"
    "setResultFormatScientific\0"
    "setResultPrecision15Digits\0"
    "setResultPrecision2Digits\0"
    "setResultPrecision3Digits\0"
    "setResultPrecision50Digits\0"
    "setResultPrecision8Digits\0"
    "setResultPrecisionAutomatic\0"
    "setResultPrecision\0setStatusBarVisible\0"
    "setSyntaxHighlightingEnabled\0"
    "setDigitGrouping\0QAction*\0"
    "setAutoResultToClipboardEnabled\0"
    "setComplexNumbers\0setVariablesDockVisible\0"
    "setUserFunctionsDockVisible\0"
    "setWindowPositionSaveEnabled\0"
    "setWidgetsDirection\0showAboutDialog\0"
    "showStateLabel\0showFontDialog\0"
    "showLanguageChooserDialog\0showManualWindow\0"
    "showContextHelp\0showReadyMessage\0"
    "showResultFormatContextMenu\0"
    "showSessionImportDialog\0showSessionLoadDialog\0"
    "wrapSelection"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MainWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
     114,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      10,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  584,    2, 0x06 /* Public */,
       3,    0,  585,    2, 0x06 /* Public */,
       4,    0,  586,    2, 0x06 /* Public */,
       5,    0,  587,    2, 0x06 /* Public */,
       6,    0,  588,    2, 0x06 /* Public */,
       7,    0,  589,    2, 0x06 /* Public */,
       8,    0,  590,    2, 0x06 /* Public */,
       9,    0,  591,    2, 0x06 /* Public */,
      10,    0,  592,    2, 0x06 /* Public */,
      11,    0,  593,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      12,    0,  594,    2, 0x0a /* Public */,
      13,    0,  595,    2, 0x08 /* Private */,
      14,    0,  596,    2, 0x08 /* Private */,
      15,    0,  597,    2, 0x08 /* Private */,
      16,    0,  598,    2, 0x08 /* Private */,
      17,    0,  599,    2, 0x08 /* Private */,
      18,    0,  600,    2, 0x08 /* Private */,
      19,    0,  601,    2, 0x08 /* Private */,
      20,    0,  602,    2, 0x08 /* Private */,
      21,    0,  603,    2, 0x08 /* Private */,
      22,    0,  604,    2, 0x08 /* Private */,
      23,    0,  605,    2, 0x08 /* Private */,
      24,    0,  606,    2, 0x08 /* Private */,
      25,    0,  607,    2, 0x08 /* Private */,
      26,    0,  608,    2, 0x08 /* Private */,
      27,    1,  609,    2, 0x08 /* Private */,
      28,    1,  612,    2, 0x08 /* Private */,
      30,    1,  615,    2, 0x08 /* Private */,
      32,    1,  618,    2, 0x08 /* Private */,
      33,    1,  621,    2, 0x08 /* Private */,
      35,    0,  624,    2, 0x08 /* Private */,
      36,    0,  625,    2, 0x08 /* Private */,
      37,    0,  626,    2, 0x08 /* Private */,
      38,    0,  627,    2, 0x08 /* Private */,
      39,    1,  628,    2, 0x08 /* Private */,
      41,    0,  631,    2, 0x08 /* Private */,
      42,    0,  632,    2, 0x08 /* Private */,
      43,    0,  633,    2, 0x08 /* Private */,
      44,    1,  634,    2, 0x08 /* Private */,
      45,    1,  637,    2, 0x08 /* Private */,
      46,    1,  640,    2, 0x08 /* Private */,
      47,    1,  643,    2, 0x08 /* Private */,
      48,    1,  646,    2, 0x08 /* Private */,
      49,    0,  649,    2, 0x08 /* Private */,
      50,    0,  650,    2, 0x08 /* Private */,
      51,    0,  651,    2, 0x08 /* Private */,
      52,    0,  652,    2, 0x08 /* Private */,
      53,    0,  653,    2, 0x08 /* Private */,
      54,    0,  654,    2, 0x08 /* Private */,
      55,    0,  655,    2, 0x08 /* Private */,
      56,    0,  656,    2, 0x08 /* Private */,
      57,    1,  657,    2, 0x08 /* Private */,
      58,    0,  660,    2, 0x08 /* Private */,
      59,    0,  661,    2, 0x08 /* Private */,
      60,    1,  662,    2, 0x08 /* Private */,
      61,    1,  665,    2, 0x08 /* Private */,
      62,    1,  668,    2, 0x08 /* Private */,
      63,    1,  671,    2, 0x08 /* Private */,
      64,    2,  674,    2, 0x08 /* Private */,
      64,    1,  679,    2, 0x28 /* Private | MethodCloned */,
      66,    2,  682,    2, 0x08 /* Private */,
      66,    1,  687,    2, 0x28 /* Private | MethodCloned */,
      67,    1,  690,    2, 0x08 /* Private */,
      68,    2,  693,    2, 0x08 /* Private */,
      68,    1,  698,    2, 0x28 /* Private | MethodCloned */,
      69,    2,  701,    2, 0x08 /* Private */,
      69,    1,  706,    2, 0x28 /* Private | MethodCloned */,
      70,    1,  709,    2, 0x08 /* Private */,
      71,    1,  712,    2, 0x08 /* Private */,
      72,    1,  715,    2, 0x08 /* Private */,
      73,    0,  718,    2, 0x08 /* Private */,
      74,    1,  719,    2, 0x08 /* Private */,
      75,    0,  722,    2, 0x08 /* Private */,
      76,    0,  723,    2, 0x08 /* Private */,
      77,    0,  724,    2, 0x08 /* Private */,
      78,    0,  725,    2, 0x08 /* Private */,
      79,    0,  726,    2, 0x08 /* Private */,
      80,    1,  727,    2, 0x08 /* Private */,
      81,    0,  730,    2, 0x08 /* Private */,
      82,    0,  731,    2, 0x08 /* Private */,
      83,    0,  732,    2, 0x08 /* Private */,
      84,    0,  733,    2, 0x08 /* Private */,
      85,    0,  734,    2, 0x08 /* Private */,
      86,    0,  735,    2, 0x08 /* Private */,
      87,    0,  736,    2, 0x08 /* Private */,
      88,    0,  737,    2, 0x08 /* Private */,
      89,    0,  738,    2, 0x08 /* Private */,
      90,    0,  739,    2, 0x08 /* Private */,
      91,    0,  740,    2, 0x08 /* Private */,
      92,    0,  741,    2, 0x08 /* Private */,
      93,    0,  742,    2, 0x08 /* Private */,
      94,    1,  743,    2, 0x08 /* Private */,
      95,    1,  746,    2, 0x08 /* Private */,
      96,    1,  749,    2, 0x08 /* Private */,
      97,    1,  752,    2, 0x08 /* Private */,
      99,    1,  755,    2, 0x08 /* Private */,
     100,    1,  758,    2, 0x08 /* Private */,
     101,    2,  761,    2, 0x08 /* Private */,
     101,    1,  766,    2, 0x28 /* Private | MethodCloned */,
     102,    2,  769,    2, 0x08 /* Private */,
     102,    1,  774,    2, 0x28 /* Private | MethodCloned */,
     103,    1,  777,    2, 0x08 /* Private */,
     104,    0,  780,    2, 0x08 /* Private */,
     105,    0,  781,    2, 0x08 /* Private */,
     106,    1,  782,    2, 0x08 /* Private */,
     107,    0,  785,    2, 0x08 /* Private */,
     108,    0,  786,    2, 0x08 /* Private */,
     109,    0,  787,    2, 0x08 /* Private */,
     110,    0,  788,    2, 0x08 /* Private */,
     111,    0,  789,    2, 0x08 /* Private */,
     112,    1,  790,    2, 0x08 /* Private */,
     113,    0,  793,    2, 0x08 /* Private */,
     114,    0,  794,    2, 0x08 /* Private */,
     115,    0,  795,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, 0x80000000 | 29,    2,
    QMetaType::Void, QMetaType::Bool,   31,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, 0x80000000 | 34,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   40,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool, QMetaType::Bool,    2,   65,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool, QMetaType::Bool,    2,   65,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool, QMetaType::Bool,    2,   65,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool, QMetaType::Bool,    2,   65,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Char,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Char,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, 0x80000000 | 98,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool, QMetaType::Bool,    2,   65,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool, QMetaType::Bool,    2,   65,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPoint,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MainWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->angleUnitChanged(); break;
        case 1: _t->colorSchemeChanged(); break;
        case 2: _t->languageChanged(); break;
        case 3: _t->radixCharacterChanged(); break;
        case 4: _t->resultFormatChanged(); break;
        case 5: _t->resultPrecisionChanged(); break;
        case 6: _t->syntaxHighlightingChanged(); break;
        case 7: _t->historyChanged(); break;
        case 8: _t->variablesChanged(); break;
        case 9: _t->functionsChanged(); break;
        case 10: _t->copy(); break;
        case 11: _t->applySelectedColorScheme(); break;
        case 12: _t->clearEditor(); break;
        case 13: _t->clearEditorAndBitfield(); break;
        case 14: _t->clearHistory(); break;
        case 15: _t->copyResultToClipboard(); break;
        case 16: _t->cycleAngleUnits(); break;
        case 17: _t->cycleResultFormats(); break;
        case 18: _t->decreaseDisplayFontPointSize(); break;
        case 19: _t->decreaseOpacity(); break;
        case 20: _t->deleteVariables(); break;
        case 21: _t->deleteUserFunctions(); break;
        case 22: _t->evaluateEditorExpression(); break;
        case 23: _t->exportHtml(); break;
        case 24: _t->exportPlainText(); break;
        case 25: _t->handleAutoCalcMessageAvailable((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 26: _t->handleAutoCalcQuantityAvailable((*reinterpret_cast< const Quantity(*)>(_a[1]))); break;
        case 27: _t->handleCopyAvailable((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 28: _t->handleBitsChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 29: _t->handleKeypadButtonPress((*reinterpret_cast< Keypad::Button(*)>(_a[1]))); break;
        case 30: _t->handleEditorTextChange(); break;
        case 31: _t->handleDisplaySelectionChange(); break;
        case 32: _t->handleEditorSelectionChange(); break;
        case 33: _t->handleManualClosed(); break;
        case 34: _t->handleDockWidgetVisibilityChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 35: _t->hideStateLabel(); break;
        case 36: _t->increaseDisplayFontPointSize(); break;
        case 37: _t->increaseOpacity(); break;
        case 38: _t->insertConstantIntoEditor((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 39: _t->insertFunctionIntoEditor((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 40: _t->insertTextIntoEditor((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 41: _t->insertVariableIntoEditor((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 42: _t->insertUserFunctionIntoEditor((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 43: _t->openUpdatesURL(); break;
        case 44: _t->openFeedbackURL(); break;
        case 45: _t->openCommunityURL(); break;
        case 46: _t->openNewsURL(); break;
        case 47: _t->openDonateURL(); break;
        case 48: _t->retranslateText(); break;
        case 49: _t->saveSessionDialog(); break;
        case 50: _t->selectEditorExpression(); break;
        case 51: _t->setAlwaysOnTopEnabled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 52: _t->setAngleModeDegree(); break;
        case 53: _t->setAngleModeRadian(); break;
        case 54: _t->setAutoAnsEnabled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 55: _t->setAutoCalcEnabled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 56: _t->setAutoCompletionEnabled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 57: _t->setBitfieldVisible((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 58: _t->setConstantsDockVisible((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 59: _t->setConstantsDockVisible((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 60: _t->setFormulaBookDockVisible((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 61: _t->setFormulaBookDockVisible((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 62: _t->setFullScreenEnabled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 63: _t->setFunctionsDockVisible((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 64: _t->setFunctionsDockVisible((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 65: _t->setHistoryDockVisible((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 66: _t->setHistoryDockVisible((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 67: _t->setSessionSaveEnabled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 68: _t->setKeypadVisible((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 69: _t->setLeaveLastExpressionEnabled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 70: _t->setRadixCharacterAutomatic(); break;
        case 71: _t->setRadixCharacter((*reinterpret_cast< char(*)>(_a[1]))); break;
        case 72: _t->setRadixCharacterComma(); break;
        case 73: _t->setRadixCharacterDot(); break;
        case 74: _t->setRadixCharacterBoth(); break;
        case 75: _t->setResultFormatBinary(); break;
        case 76: _t->setResultFormatCartesian(); break;
        case 77: _t->setResultFormat((*reinterpret_cast< char(*)>(_a[1]))); break;
        case 78: _t->setResultFormatEngineering(); break;
        case 79: _t->setResultFormatFixed(); break;
        case 80: _t->setResultFormatGeneral(); break;
        case 81: _t->setResultFormatHexadecimal(); break;
        case 82: _t->setResultFormatOctal(); break;
        case 83: _t->setResultFormatPolar(); break;
        case 84: _t->setResultFormatScientific(); break;
        case 85: _t->setResultPrecision15Digits(); break;
        case 86: _t->setResultPrecision2Digits(); break;
        case 87: _t->setResultPrecision3Digits(); break;
        case 88: _t->setResultPrecision50Digits(); break;
        case 89: _t->setResultPrecision8Digits(); break;
        case 90: _t->setResultPrecisionAutomatic(); break;
        case 91: _t->setResultPrecision((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 92: _t->setStatusBarVisible((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 93: _t->setSyntaxHighlightingEnabled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 94: _t->setDigitGrouping((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 95: _t->setAutoResultToClipboardEnabled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 96: _t->setComplexNumbers((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 97: _t->setVariablesDockVisible((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 98: _t->setVariablesDockVisible((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 99: _t->setUserFunctionsDockVisible((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 100: _t->setUserFunctionsDockVisible((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 101: _t->setWindowPositionSaveEnabled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 102: _t->setWidgetsDirection(); break;
        case 103: _t->showAboutDialog(); break;
        case 104: _t->showStateLabel((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 105: _t->showFontDialog(); break;
        case 106: _t->showLanguageChooserDialog(); break;
        case 107: _t->showManualWindow(); break;
        case 108: _t->showContextHelp(); break;
        case 109: _t->showReadyMessage(); break;
        case 110: _t->showResultFormatContextMenu((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 111: _t->showSessionImportDialog(); break;
        case 112: _t->showSessionLoadDialog(); break;
        case 113: _t->wrapSelection(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (MainWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MainWindow::angleUnitChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (MainWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MainWindow::colorSchemeChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (MainWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MainWindow::languageChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (MainWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MainWindow::radixCharacterChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (MainWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MainWindow::resultFormatChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (MainWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MainWindow::resultPrecisionChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (MainWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MainWindow::syntaxHighlightingChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (MainWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MainWindow::historyChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (MainWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MainWindow::variablesChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (MainWindow::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MainWindow::functionsChanged)) {
                *result = 9;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_MainWindow.data,
    qt_meta_data_MainWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MainWindow.stringdata0))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 114)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 114;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 114)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 114;
    }
    return _id;
}

// SIGNAL 0
void MainWindow::angleUnitChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void MainWindow::colorSchemeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void MainWindow::languageChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void MainWindow::radixCharacterChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void MainWindow::resultFormatChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void MainWindow::resultPrecisionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void MainWindow::syntaxHighlightingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void MainWindow::historyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void MainWindow::variablesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void MainWindow::functionsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
