/****************************************************************************
** Meta object code from reading C++ file 'editor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../speedcrunch/src/gui/editor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'editor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Editor_t {
    QByteArrayData data[41];
    char stringdata0[647];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Editor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Editor_t qt_meta_stringdata_Editor = {
    {
QT_MOC_LITERAL(0, 0, 6), // "Editor"
QT_MOC_LITERAL(1, 7, 24), // "autoCalcMessageAvailable"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 25), // "autoCalcQuantityAvailable"
QT_MOC_LITERAL(4, 59, 8), // "Quantity"
QT_MOC_LITERAL(5, 68, 16), // "autoCalcDisabled"
QT_MOC_LITERAL(6, 85, 22), // "controlPageDownPressed"
QT_MOC_LITERAL(7, 108, 20), // "controlPageUpPressed"
QT_MOC_LITERAL(8, 129, 19), // "copySequencePressed"
QT_MOC_LITERAL(9, 149, 15), // "pageDownPressed"
QT_MOC_LITERAL(10, 165, 13), // "pageUpPressed"
QT_MOC_LITERAL(11, 179, 13), // "returnPressed"
QT_MOC_LITERAL(12, 193, 16), // "shiftDownPressed"
QT_MOC_LITERAL(13, 210, 14), // "shiftUpPressed"
QT_MOC_LITERAL(14, 225, 20), // "shiftPageDownPressed"
QT_MOC_LITERAL(15, 246, 18), // "shiftPageUpPressed"
QT_MOC_LITERAL(16, 265, 17), // "autoCalcSelection"
QT_MOC_LITERAL(17, 283, 6), // "custom"
QT_MOC_LITERAL(18, 290, 24), // "cancelConstantCompletion"
QT_MOC_LITERAL(19, 315, 8), // "evaluate"
QT_MOC_LITERAL(20, 324, 21), // "decreaseFontPointSize"
QT_MOC_LITERAL(21, 346, 21), // "increaseFontPointSize"
QT_MOC_LITERAL(22, 368, 6), // "insert"
QT_MOC_LITERAL(23, 375, 14), // "insertConstant"
QT_MOC_LITERAL(24, 390, 11), // "rehighlight"
QT_MOC_LITERAL(25, 402, 13), // "updateHistory"
QT_MOC_LITERAL(26, 416, 18), // "insertFromMimeData"
QT_MOC_LITERAL(27, 435, 16), // "const QMimeData*"
QT_MOC_LITERAL(28, 452, 8), // "autoCalc"
QT_MOC_LITERAL(29, 461, 12), // "autoComplete"
QT_MOC_LITERAL(30, 474, 13), // "checkAutoCalc"
QT_MOC_LITERAL(31, 488, 17), // "checkAutoComplete"
QT_MOC_LITERAL(32, 506, 13), // "checkMatching"
QT_MOC_LITERAL(33, 520, 14), // "doMatchingLeft"
QT_MOC_LITERAL(34, 535, 13), // "doMatchingPar"
QT_MOC_LITERAL(35, 549, 15), // "doMatchingRight"
QT_MOC_LITERAL(36, 565, 11), // "historyBack"
QT_MOC_LITERAL(37, 577, 14), // "historyForward"
QT_MOC_LITERAL(38, 592, 21), // "startSelAutoCalcTimer"
QT_MOC_LITERAL(39, 614, 19), // "triggerAutoComplete"
QT_MOC_LITERAL(40, 634, 12) // "triggerEnter"

    },
    "Editor\0autoCalcMessageAvailable\0\0"
    "autoCalcQuantityAvailable\0Quantity\0"
    "autoCalcDisabled\0controlPageDownPressed\0"
    "controlPageUpPressed\0copySequencePressed\0"
    "pageDownPressed\0pageUpPressed\0"
    "returnPressed\0shiftDownPressed\0"
    "shiftUpPressed\0shiftPageDownPressed\0"
    "shiftPageUpPressed\0autoCalcSelection\0"
    "custom\0cancelConstantCompletion\0"
    "evaluate\0decreaseFontPointSize\0"
    "increaseFontPointSize\0insert\0"
    "insertConstant\0rehighlight\0updateHistory\0"
    "insertFromMimeData\0const QMimeData*\0"
    "autoCalc\0autoComplete\0checkAutoCalc\0"
    "checkAutoComplete\0checkMatching\0"
    "doMatchingLeft\0doMatchingPar\0"
    "doMatchingRight\0historyBack\0historyForward\0"
    "startSelAutoCalcTimer\0triggerAutoComplete\0"
    "triggerEnter"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Editor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      37,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      13,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  199,    2, 0x06 /* Public */,
       3,    1,  202,    2, 0x06 /* Public */,
       5,    0,  205,    2, 0x06 /* Public */,
       6,    0,  206,    2, 0x06 /* Public */,
       7,    0,  207,    2, 0x06 /* Public */,
       8,    0,  208,    2, 0x06 /* Public */,
       9,    0,  209,    2, 0x06 /* Public */,
      10,    0,  210,    2, 0x06 /* Public */,
      11,    0,  211,    2, 0x06 /* Public */,
      12,    0,  212,    2, 0x06 /* Public */,
      13,    0,  213,    2, 0x06 /* Public */,
      14,    0,  214,    2, 0x06 /* Public */,
      15,    0,  215,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      16,    1,  216,    2, 0x0a /* Public */,
      16,    0,  219,    2, 0x2a /* Public | MethodCloned */,
      18,    0,  220,    2, 0x0a /* Public */,
      19,    0,  221,    2, 0x0a /* Public */,
      20,    0,  222,    2, 0x0a /* Public */,
      21,    0,  223,    2, 0x0a /* Public */,
      22,    1,  224,    2, 0x0a /* Public */,
      23,    1,  227,    2, 0x0a /* Public */,
      24,    0,  230,    2, 0x0a /* Public */,
      25,    0,  231,    2, 0x0a /* Public */,
      26,    1,  232,    2, 0x09 /* Protected */,
      28,    0,  235,    2, 0x09 /* Protected */,
      29,    1,  236,    2, 0x09 /* Protected */,
      30,    0,  239,    2, 0x09 /* Protected */,
      31,    0,  240,    2, 0x09 /* Protected */,
      32,    0,  241,    2, 0x09 /* Protected */,
      33,    0,  242,    2, 0x09 /* Protected */,
      34,    0,  243,    2, 0x09 /* Protected */,
      35,    0,  244,    2, 0x09 /* Protected */,
      36,    0,  245,    2, 0x09 /* Protected */,
      37,    0,  246,    2, 0x09 /* Protected */,
      38,    0,  247,    2, 0x09 /* Protected */,
      39,    0,  248,    2, 0x09 /* Protected */,
      40,    0,  249,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,   17,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 27,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Editor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Editor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->autoCalcMessageAvailable((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->autoCalcQuantityAvailable((*reinterpret_cast< const Quantity(*)>(_a[1]))); break;
        case 2: _t->autoCalcDisabled(); break;
        case 3: _t->controlPageDownPressed(); break;
        case 4: _t->controlPageUpPressed(); break;
        case 5: _t->copySequencePressed(); break;
        case 6: _t->pageDownPressed(); break;
        case 7: _t->pageUpPressed(); break;
        case 8: _t->returnPressed(); break;
        case 9: _t->shiftDownPressed(); break;
        case 10: _t->shiftUpPressed(); break;
        case 11: _t->shiftPageDownPressed(); break;
        case 12: _t->shiftPageUpPressed(); break;
        case 13: _t->autoCalcSelection((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 14: _t->autoCalcSelection(); break;
        case 15: _t->cancelConstantCompletion(); break;
        case 16: _t->evaluate(); break;
        case 17: _t->decreaseFontPointSize(); break;
        case 18: _t->increaseFontPointSize(); break;
        case 19: _t->insert((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 20: _t->insertConstant((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 21: _t->rehighlight(); break;
        case 22: _t->updateHistory(); break;
        case 23: _t->insertFromMimeData((*reinterpret_cast< const QMimeData*(*)>(_a[1]))); break;
        case 24: _t->autoCalc(); break;
        case 25: _t->autoComplete((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 26: _t->checkAutoCalc(); break;
        case 27: _t->checkAutoComplete(); break;
        case 28: _t->checkMatching(); break;
        case 29: _t->doMatchingLeft(); break;
        case 30: _t->doMatchingPar(); break;
        case 31: _t->doMatchingRight(); break;
        case 32: _t->historyBack(); break;
        case 33: _t->historyForward(); break;
        case 34: _t->startSelAutoCalcTimer(); break;
        case 35: _t->triggerAutoComplete(); break;
        case 36: _t->triggerEnter(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Editor::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Editor::autoCalcMessageAvailable)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Editor::*)(const Quantity & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Editor::autoCalcQuantityAvailable)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Editor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Editor::autoCalcDisabled)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Editor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Editor::controlPageDownPressed)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (Editor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Editor::controlPageUpPressed)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (Editor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Editor::copySequencePressed)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (Editor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Editor::pageDownPressed)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (Editor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Editor::pageUpPressed)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (Editor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Editor::returnPressed)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (Editor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Editor::shiftDownPressed)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (Editor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Editor::shiftUpPressed)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (Editor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Editor::shiftPageDownPressed)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (Editor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Editor::shiftPageUpPressed)) {
                *result = 12;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Editor::staticMetaObject = { {
    QMetaObject::SuperData::link<QPlainTextEdit::staticMetaObject>(),
    qt_meta_stringdata_Editor.data,
    qt_meta_data_Editor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Editor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Editor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Editor.stringdata0))
        return static_cast<void*>(this);
    return QPlainTextEdit::qt_metacast(_clname);
}

int Editor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QPlainTextEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 37)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 37;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 37)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 37;
    }
    return _id;
}

// SIGNAL 0
void Editor::autoCalcMessageAvailable(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Editor::autoCalcQuantityAvailable(const Quantity & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Editor::autoCalcDisabled()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Editor::controlPageDownPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Editor::controlPageUpPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Editor::copySequencePressed()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Editor::pageDownPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Editor::pageUpPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Editor::returnPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void Editor::shiftDownPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void Editor::shiftUpPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void Editor::shiftPageDownPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void Editor::shiftPageUpPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}
struct qt_meta_stringdata_EditorCompletion_t {
    QByteArrayData data[5];
    char stringdata0[63];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_EditorCompletion_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_EditorCompletion_t qt_meta_stringdata_EditorCompletion = {
    {
QT_MOC_LITERAL(0, 0, 16), // "EditorCompletion"
QT_MOC_LITERAL(1, 17, 18), // "selectedCompletion"
QT_MOC_LITERAL(2, 36, 0), // ""
QT_MOC_LITERAL(3, 37, 14), // "doneCompletion"
QT_MOC_LITERAL(4, 52, 10) // "selectItem"

    },
    "EditorCompletion\0selectedCompletion\0"
    "\0doneCompletion\0selectItem"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_EditorCompletion[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   29,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,   32,    2, 0x0a /* Public */,
       4,    1,   33,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,

       0        // eod
};

void EditorCompletion::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<EditorCompletion *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->selectedCompletion((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->doneCompletion(); break;
        case 2: _t->selectItem((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (EditorCompletion::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&EditorCompletion::selectedCompletion)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject EditorCompletion::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_EditorCompletion.data,
    qt_meta_data_EditorCompletion,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *EditorCompletion::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *EditorCompletion::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_EditorCompletion.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int EditorCompletion::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void EditorCompletion::selectedCompletion(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_ConstantCompletion_t {
    QByteArrayData data[8];
    char stringdata0[122];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ConstantCompletion_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ConstantCompletion_t qt_meta_stringdata_ConstantCompletion = {
    {
QT_MOC_LITERAL(0, 0, 18), // "ConstantCompletion"
QT_MOC_LITERAL(1, 19, 18), // "canceledCompletion"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 18), // "selectedCompletion"
QT_MOC_LITERAL(4, 58, 14), // "doneCompletion"
QT_MOC_LITERAL(5, 73, 21), // "setHorizontalPosition"
QT_MOC_LITERAL(6, 95, 12), // "showCategory"
QT_MOC_LITERAL(7, 108, 13) // "showConstants"

    },
    "ConstantCompletion\0canceledCompletion\0"
    "\0selectedCompletion\0doneCompletion\0"
    "setHorizontalPosition\0showCategory\0"
    "showConstants"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ConstantCompletion[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   44,    2, 0x06 /* Public */,
       3,    1,   45,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   48,    2, 0x0a /* Public */,
       5,    1,   49,    2, 0x09 /* Protected */,
       6,    0,   52,    2, 0x09 /* Protected */,
       7,    0,   53,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ConstantCompletion::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ConstantCompletion *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->canceledCompletion(); break;
        case 1: _t->selectedCompletion((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->doneCompletion(); break;
        case 3: _t->setHorizontalPosition((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->showCategory(); break;
        case 5: _t->showConstants(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ConstantCompletion::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ConstantCompletion::canceledCompletion)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ConstantCompletion::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ConstantCompletion::selectedCompletion)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ConstantCompletion::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ConstantCompletion.data,
    qt_meta_data_ConstantCompletion,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ConstantCompletion::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ConstantCompletion::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ConstantCompletion.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ConstantCompletion::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void ConstantCompletion::canceledCompletion()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void ConstantCompletion::selectedCompletion(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
