/****************************************************************************
** Meta object code from reading C++ file 'resultdisplay.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../speedcrunch/src/gui/resultdisplay.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'resultdisplay.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ResultDisplay_t {
    QByteArrayData data[21];
    char stringdata0[301];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ResultDisplay_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ResultDisplay_t qt_meta_stringdata_ResultDisplay = {
    {
QT_MOC_LITERAL(0, 0, 13), // "ResultDisplay"
QT_MOC_LITERAL(1, 14, 14), // "shiftWheelDown"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 12), // "shiftWheelUp"
QT_MOC_LITERAL(4, 43, 21), // "shiftControlWheelDown"
QT_MOC_LITERAL(5, 65, 19), // "shiftControlWheelUp"
QT_MOC_LITERAL(6, 85, 16), // "controlWheelDown"
QT_MOC_LITERAL(7, 102, 14), // "controlWheelUp"
QT_MOC_LITERAL(8, 117, 18), // "expressionSelected"
QT_MOC_LITERAL(9, 136, 5), // "clear"
QT_MOC_LITERAL(10, 142, 21), // "decreaseFontPointSize"
QT_MOC_LITERAL(11, 164, 21), // "increaseFontPointSize"
QT_MOC_LITERAL(12, 186, 11), // "rehighlight"
QT_MOC_LITERAL(13, 198, 7), // "refresh"
QT_MOC_LITERAL(14, 206, 11), // "scrollLines"
QT_MOC_LITERAL(15, 218, 12), // "scrollLineUp"
QT_MOC_LITERAL(16, 231, 14), // "scrollLineDown"
QT_MOC_LITERAL(17, 246, 12), // "scrollPageUp"
QT_MOC_LITERAL(18, 259, 14), // "scrollPageDown"
QT_MOC_LITERAL(19, 274, 14), // "scrollToBottom"
QT_MOC_LITERAL(20, 289, 11) // "scrollToTop"

    },
    "ResultDisplay\0shiftWheelDown\0\0"
    "shiftWheelUp\0shiftControlWheelDown\0"
    "shiftControlWheelUp\0controlWheelDown\0"
    "controlWheelUp\0expressionSelected\0"
    "clear\0decreaseFontPointSize\0"
    "increaseFontPointSize\0rehighlight\0"
    "refresh\0scrollLines\0scrollLineUp\0"
    "scrollLineDown\0scrollPageUp\0scrollPageDown\0"
    "scrollToBottom\0scrollToTop"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ResultDisplay[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      19,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  109,    2, 0x06 /* Public */,
       3,    0,  110,    2, 0x06 /* Public */,
       4,    0,  111,    2, 0x06 /* Public */,
       5,    0,  112,    2, 0x06 /* Public */,
       6,    0,  113,    2, 0x06 /* Public */,
       7,    0,  114,    2, 0x06 /* Public */,
       8,    1,  115,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    0,  118,    2, 0x0a /* Public */,
      10,    0,  119,    2, 0x0a /* Public */,
      11,    0,  120,    2, 0x0a /* Public */,
      12,    0,  121,    2, 0x0a /* Public */,
      13,    0,  122,    2, 0x0a /* Public */,
      14,    1,  123,    2, 0x0a /* Public */,
      15,    0,  126,    2, 0x0a /* Public */,
      16,    0,  127,    2, 0x0a /* Public */,
      17,    0,  128,    2, 0x0a /* Public */,
      18,    0,  129,    2, 0x0a /* Public */,
      19,    0,  130,    2, 0x0a /* Public */,
      20,    0,  131,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ResultDisplay::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ResultDisplay *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->shiftWheelDown(); break;
        case 1: _t->shiftWheelUp(); break;
        case 2: _t->shiftControlWheelDown(); break;
        case 3: _t->shiftControlWheelUp(); break;
        case 4: _t->controlWheelDown(); break;
        case 5: _t->controlWheelUp(); break;
        case 6: _t->expressionSelected((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->clear(); break;
        case 8: _t->decreaseFontPointSize(); break;
        case 9: _t->increaseFontPointSize(); break;
        case 10: _t->rehighlight(); break;
        case 11: _t->refresh(); break;
        case 12: _t->scrollLines((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->scrollLineUp(); break;
        case 14: _t->scrollLineDown(); break;
        case 15: _t->scrollPageUp(); break;
        case 16: _t->scrollPageDown(); break;
        case 17: _t->scrollToBottom(); break;
        case 18: _t->scrollToTop(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ResultDisplay::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ResultDisplay::shiftWheelDown)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ResultDisplay::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ResultDisplay::shiftWheelUp)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (ResultDisplay::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ResultDisplay::shiftControlWheelDown)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (ResultDisplay::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ResultDisplay::shiftControlWheelUp)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (ResultDisplay::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ResultDisplay::controlWheelDown)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (ResultDisplay::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ResultDisplay::controlWheelUp)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (ResultDisplay::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ResultDisplay::expressionSelected)) {
                *result = 6;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ResultDisplay::staticMetaObject = { {
    QMetaObject::SuperData::link<QPlainTextEdit::staticMetaObject>(),
    qt_meta_stringdata_ResultDisplay.data,
    qt_meta_data_ResultDisplay,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ResultDisplay::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ResultDisplay::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ResultDisplay.stringdata0))
        return static_cast<void*>(this);
    return QPlainTextEdit::qt_metacast(_clname);
}

int ResultDisplay::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QPlainTextEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void ResultDisplay::shiftWheelDown()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void ResultDisplay::shiftWheelUp()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void ResultDisplay::shiftControlWheelDown()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void ResultDisplay::shiftControlWheelUp()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void ResultDisplay::controlWheelDown()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void ResultDisplay::controlWheelUp()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void ResultDisplay::expressionSelected(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
