// This file is part of the SpeedCrunch project
// Copyright (C) 2016 Pol Welter <polwelter@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
// Boston, MA 02110-1301, USA.

#ifndef CORE_MANUALSERVER_H
#define CORE_MANUALSERVER_H

#include <QObject>
#include <QMap>

class QHelpEngineCore;

class QCloseEvent;
class QUrl;
class QString;
class QByteArray;

class ManualServer : public QObject {
    Q_OBJECT

private:
    QString deployDocs();
    void setupHelpEngine();

public:
    static ManualServer* instance();
    bool URLforKeyword(const QString id, QUrl &result);
    QByteArray fileData(const QUrl &url);
    bool isSupportedLanguage(const QString&);

public slots:
    void ensureCorrectLanguage();

private:
    ManualServer();
    Q_DISABLE_COPY(ManualServer)

    void languageChanged();

    QHelpEngineCore *m_helpEngine;
    static ManualServer* s_instance;
    QString m_deployedLanguage;
};

#endif // CORE_MANUALSERVER_H
